@@ -20,7 +20,7 @@ def set_base_parser():
         'Examples': ('🚀‍', 'https://learn.jina.ai'),
         'Dashboard': ('📊', 'https://dashboard.jina.ai'),
         'Code': ('🧑‍💻', 'https://opensource.jina.ai'),
-        'Hiring!': ('🙌', 'hello@jina.ai')
+        'Hiring!': ('🙌', 'career@jina.ai')
     }
     url_str = '\n'.join(f'{v[0]} {k:10.10} {colored(v[1], "cyan", attrs=["underline"])}' for k, v in urls.items())
 
@@ -214,6 +214,14 @@ def set_pea_parser(parser=None):
     gp5.add_argument('--array-in-pb', action='store_true', default=False,
                      help='sending raw_bytes and numpy ndarray together within or separately from the protobuf message, '
                           'the latter often yields a better network efficiency')
+    gp5.add_argument('--compress-hwm', type=int, default=-1,
+                     help='the high watermark that triggers the message compression. '
+                          'message bigger than this HWM (in bytes) will be compressed by lz4 algorithm.'
+                          'set this to -1 to disable this feature.')
+    gp5.add_argument('--compress-lwm', type=float, default=1.,
+                     help='the low watermark that enables the sending of a compressed message. '
+                          'compression rate (after_size/before_size) lower than this LWM will considered as successeful '
+                          'compression, and will be sent. Otherwise, it will send the original message without compression')
     gp5.add_argument('--num-part', type=int, default=1,
                      help='wait until the number of parts of message are all received')
 
@@ -307,6 +315,17 @@ def set_check_parser(parser=None):
     return parser
 
 
+def set_export_api_parser(parser=None):
+    if not parser:
+        parser = set_base_parser()
+
+    parser.add_argument('--yaml-path', type=str, nargs='*',
+                        help='the YAML file path for storing the exported API')
+    parser.add_argument('--json-path', type=str, nargs='*',
+                        help='the JSON file path for storing the exported API')
+    return parser
+
+
 def _set_grpc_parser(parser=None):
     if not parser:
         parser = set_base_parser()
@@ -460,6 +479,11 @@ def get_main_parser():
         sp.add_parser('client',
                       description='Start a Python client that connects to a remote Jina gateway',
                       formatter_class=_chf, **(dict(help='start a client')) if show_all else {}))
+
+    set_export_api_parser(sp.add_parser('export-api',
+                                        description='Export Jina API to JSON/YAML file for 3rd party applications',
+                                        formatter_class=_chf,
+                                        **(dict(help='export Jina API to file')) if show_all else {}))
     return parser
 
 
